@@ -1,7 +1,7 @@
 package processing
 
 // DCSO FEVER
-// Copyright (c) 2017, 2018, DCSO GmbH
+// Copyright (c) 2017, 2018, 2019, DCSO GmbH
 
 import (
 	"encoding/json"
@@ -19,12 +19,14 @@ import (
 
 	"github.com/DCSO/bloom"
 	log "github.com/sirupsen/logrus"
+	"github.com/sirupsen/logrus/hooks/test"
 )
 
 var (
 	reHTTPURL  = regexp.MustCompile(`Possibly bad HTTP URL: [^ ]+ . ([^ ]+) . ([^" ]+)`)
 	reHTTPHost = regexp.MustCompile(`Possibly bad HTTP host: ([^" ]+)`)
-	reDNS      = regexp.MustCompile("Possibly bad DNS lookup to ([^\" ]+)")
+	reDNSReq   = regexp.MustCompile("Possibly bad DNS lookup to ([^\" ]+)")
+	reDNSRep   = regexp.MustCompile("Possibly bad DNS response for ([^\" ]+)")
 	reSNI      = regexp.MustCompile("Possibly bad TLS SNI: ([^\" ]+)")
 )
 
@@ -40,7 +42,8 @@ func makeBloomDNSEvent(rrname string) types.Entry {
 		DNSRCode:  []string{"NOERROR", "NXDOMAIN"}[rand.Intn(2)],
 		DNSRData:  fmt.Sprintf("10.%d.0.%d", rand.Intn(50), rand.Intn(50)+100),
 		DNSRRName: rrname,
-		DNSRRType: "answer",
+		DNSRRType: "A",
+		DNSType:   []string{"answer", "query"}[rand.Intn(2)],
 	}
 	eve := types.EveEvent{
 		EventType: e.EventType,
@@ -54,6 +57,7 @@ func makeBloomDNSEvent(rrname string) types.Entry {
 			Rrname: e.DNSRRName,
 			Rdata:  e.DNSRData,
 			Rrtype: e.DNSRRType,
+			Type:   e.DNSType,
 		},
 	}
 	json, err := json.Marshal(eve)
@@ -202,8 +206,29 @@ func (h *CollectorHandler) Consume(e *types.Entry) error {
 		h.Entries[host] = true
 		return nil
 	}
-	match = reDNS.FindStringSubmatch(e.JSONLine)
+	match = reDNSReq.FindStringSubmatch(e.JSONLine)
 	if match != nil {
+		var eve types.EveEvent
+		var err = json.Unmarshal([]byte(e.JSONLine), &eve)
+		if err != nil {
+			log.Fatal(err)
+		}
+		if eve.DNS.Type != "query" {
+			log.Fatalf("request alert for type (%s) != query", eve.DNS.Type)
+		}
+		h.Entries[match[1]] = true
+		return nil
+	}
+	match = reDNSRep.FindStringSubmatch(e.JSONLine)
+	if match != nil {
+		var eve types.EveEvent
+		var err = json.Unmarshal([]byte(e.JSONLine), &eve)
+		if err != nil {
+			log.Fatal(err)
+		}
+		if eve.DNS.Type != "answer" {
+			log.Fatalf("request alert for type (%s) != answer", eve.DNS.Type)
+		}
 		h.Entries[match[1]] = true
 		return nil
 	}
@@ -388,7 +413,7 @@ func TestBloomHandlerFromFile(t *testing.T) {
 	dbChan := make(chan types.Entry, 10)
 	defer close(dbChan)
 
-	bh, err := MakeBloomHandlerFromFile(b1File.Name(), false, dbChan, fwhandler, "FOO BAR")
+	bh, err := MakeBloomHandlerFromFile(b1File.Name(), false, dbChan, fwhandler, "FOO BAR", []string{"/"})
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -457,7 +482,7 @@ func TestBloomHandlerEmptyInput(t *testing.T) {
 	dbChan := make(chan types.Entry, 10)
 	defer close(dbChan)
 
-	bf, err := MakeBloomHandlerFromFile(blFile.Name(), false, dbChan, nil, "FOO BAR")
+	bf, err := MakeBloomHandlerFromFile(blFile.Name(), false, dbChan, nil, "FOO BAR", []string{"/"})
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -466,6 +491,69 @@ func TestBloomHandlerEmptyInput(t *testing.T) {
 	}
 }
 
+func TestBloomHandlerBlacklistedInputFromFile(t *testing.T) {
+	b1 := bloom.Initialize(1000, 0.0001)
+	b1.Add([]byte("/"))
+	b1File, err := ioutil.TempFile("", "blist")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer os.Remove(b1File.Name())
+	b1.Write(b1File)
+	b1File.Close()
+
+	b2 := bloom.Initialize(1000, 0.0001)
+	b2.Add([]byte("/foobarbaz"))
+
+	dbChan := make(chan types.Entry, 10)
+	defer close(dbChan)
+
+	hook := test.NewGlobal()
+	_, err = MakeBloomHandlerFromFile(b1File.Name(), false, nil, nil, "FOO BAR", []string{"/"})
+	if err != nil {
+		t.Fatal(err)
+	}
+	entries := hook.AllEntries()
+	if len(entries) != 4 {
+		t.Fatal("missing log entries")
+	}
+	if entries[2].Message != "filter contains blacklisted indicator '/'" {
+		t.Fatal("wrong log entry for invalid IP range")
+	}
+
+	b2File, err := os.OpenFile(b1File.Name(), os.O_RDWR|os.O_CREATE, 0755)
+	if err != nil {
+		t.Fatal(err)
+	}
+	b2.Write(b2File)
+	b2File.Close()
+
+	bf, err := MakeBloomHandlerFromFile(b1File.Name(), false, nil, nil, "FOO BAR", []string{"/"})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	b2File, err = os.OpenFile(b1File.Name(), os.O_RDWR|os.O_CREATE, 0755)
+	if err != nil {
+		t.Fatal(err)
+	}
+	b1.Write(b2File)
+	b2File.Close()
+
+	hook.Reset()
+	err = bf.Reload()
+	if err != nil {
+		t.Fatal(err)
+	}
+	entries = hook.AllEntries()
+	if len(entries) != 2 {
+		t.Fatal("missing log entries")
+	}
+	if entries[0].Message != "filter contains blacklisted indicator '/'" {
+		t.Fatal("wrong log entry for invalid IP range")
+	}
+}
+
 func TestBloomHandlerURL(t *testing.T) {
 	e1 := types.Entry{
 		SrcIP:      "10.0.0.1",
@@ -738,5 +826,160 @@ func TestBloomHandlerURL(t *testing.T) {
 	if len(fwhandler.GetEntries()) != 0 {
 		t.Fatalf("too many alerts: %d", len(fwhandler.GetEntries()))
 	}
+}
 
+func TestBloomHandlerBlacklistedSkip(t *testing.T) {
+	e1 := types.Entry{
+		SrcIP:      "10.0.0.1",
+		SrcPort:    23545,
+		DestIP:     "10.0.0.2",
+		DestPort:   80,
+		Timestamp:  time.Now().Format(types.SuricataTimestampFormat),
+		EventType:  "http",
+		Proto:      "TCP",
+		HTTPHost:   "foo.bar.de",
+		HTTPUrl:    "http://foo.bar.de/oddlyspecific",
+		HTTPMethod: "GET",
+	}
+	eve1 := types.EveEvent{
+		EventType: e1.EventType,
+		SrcIP:     e1.SrcIP,
+		SrcPort:   int(e1.SrcPort),
+		DestIP:    e1.DestIP,
+		DestPort:  int(e1.DestPort),
+		Proto:     e1.Proto,
+		HTTP: &types.HTTPEvent{
+			Hostname: e1.HTTPHost,
+			URL:      e1.HTTPUrl,
+		},
+	}
+	json1, err := json.Marshal(eve1)
+	if err != nil {
+		log.Warn(err)
+	} else {
+		e1.JSONLine = string(json1)
+	}
+	e2 := types.Entry{
+		SrcIP:      "10.0.0.1",
+		SrcPort:    23545,
+		DestIP:     "10.0.0.2",
+		DestPort:   80,
+		Timestamp:  time.Now().Format(types.SuricataTimestampFormat),
+		EventType:  "http",
+		Proto:      "TCP",
+		HTTPHost:   "foo.bar.de",
+		HTTPUrl:    "/",
+		HTTPMethod: "GET",
+	}
+	eve2 := types.EveEvent{
+		EventType: e2.EventType,
+		SrcIP:     e2.SrcIP,
+		SrcPort:   int(e2.SrcPort),
+		DestIP:    e2.DestIP,
+		DestPort:  int(e2.DestPort),
+		Proto:     e2.Proto,
+		HTTP: &types.HTTPEvent{
+			Hostname: e2.HTTPHost,
+			URL:      e2.HTTPUrl,
+		},
+	}
+	json2, err := json.Marshal(eve2)
+	if err != nil {
+		log.Warn(err)
+	} else {
+		e2.JSONLine = string(json2)
+	}
+
+	b1 := bloom.Initialize(1000, 0.0001)
+	b1.Add([]byte("/oddlyspecific"))
+	b1.Add([]byte("/"))
+	b1File, err := ioutil.TempFile("", "blist")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer os.Remove(b1File.Name())
+	b1.Write(b1File)
+	b1File.Close()
+
+	dbChan := make(chan types.Entry, 5)
+	dbWritten := make([]types.Entry, 0)
+	consumeWaitChan := make(chan bool)
+	go func() {
+		for e := range dbChan {
+			dbWritten = append(dbWritten, e)
+		}
+		close(consumeWaitChan)
+	}()
+
+	util.PrepareEventFilter([]string{"alert"}, false)
+
+	// handler to receive forwarded events
+	fwhandler := &CollectorHandler{
+		Entries: make(map[string]bool),
+	}
+
+	bh, err := MakeBloomHandlerFromFile(b1File.Name(), false, dbChan, fwhandler, "FOO BAR", []string{"/"})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	bh.Consume(&e1)
+	if len(fwhandler.GetEntries()) != 1 {
+		t.Fatalf("not enough alerts: %d", len(fwhandler.GetEntries()))
+	}
+
+	fwhandler.Reset()
+	bh.Consume(&e2)
+
+	if len(fwhandler.GetEntries()) != 0 {
+		t.Fatalf("should not create alert but got %d", len(fwhandler.GetEntries()))
+	}
+
+	bh.Consume(&e1)
+	if len(fwhandler.GetEntries()) != 1 {
+		t.Fatalf("not enough alerts: %d", len(fwhandler.GetEntries()))
+	}
+}
+
+func TestBloomHandlerInvalidDNS(t *testing.T) {
+	// make sure that alerts are forwarded
+	util.PrepareEventFilter([]string{"alert"}, false)
+
+	// initalize Bloom filter and fill with 'interesting' values
+	bf := bloom.Initialize(100000, 0.0000001)
+
+	// channel to receive events to be saved to database
+	dbChan := make(chan types.Entry)
+
+	// handler to receive forwarded events
+	fwhandler := &CollectorHandler{
+		Entries: make(map[string]bool),
+	}
+
+	// concurrently gather entries to be written to DB
+	dbWritten := make([]types.Entry, 0)
+	consumeWaitChan := make(chan bool)
+	go func() {
+		for e := range dbChan {
+			dbWritten = append(dbWritten, e)
+		}
+		close(consumeWaitChan)
+	}()
+
+	bh := MakeBloomHandler(&bf, dbChan, fwhandler, "FOO BAR")
+	e := makeBloomDNSEvent("foobar")
+	e.DNSType = "foobar"
+	bf.Add([]byte(e.DNSRRName))
+
+	hook := test.NewGlobal()
+
+	bh.Consume(&e)
+
+	entries := hook.AllEntries()
+	if len(entries) < 1 {
+		t.Fatal("missing log entries")
+	}
+	if entries[0].Message != "invalid DNS type: 'foobar'" {
+		t.Fatal("wrong log entry for invalid DNS type")
+	}
 }
