@@ -1,7 +1,7 @@
 package processing
 
 // DCSO FEVER
-// Copyright (c) 2017, DCSO GmbH
+// Copyright (c) 2017, 2019, DCSO GmbH
 
 import (
 	"encoding/json"
@@ -28,6 +28,7 @@ type ForwardHandler struct {
 	Logger              *log.Entry
 	DoRDNS              bool
 	RDNSHandler         *RDNSHandler
+	ContextCollector    *ContextCollector
 	ForwardEventChan    chan []byte
 	OutputSocket        string
 	OutputConn          net.Conn
@@ -180,11 +181,14 @@ func MakeForwardHandler(reconnectTimes int, outputSocket string) *ForwardHandler
 func (fh *ForwardHandler) Consume(e *types.Entry) error {
 	doForwardThis := util.ForwardAllEvents || util.AllowType(e.EventType)
 	if doForwardThis {
-		var ev types.EveEvent
+		var ev types.EveOutEvent
 		err := json.Unmarshal([]byte(e.JSONLine), &ev)
 		if err != nil {
 			return err
 		}
+		if GlobalContextCollector != nil && e.EventType == types.EventTypeAlert {
+			GlobalContextCollector.Mark(string(e.FlowID))
+		}
 		if fh.DoRDNS && fh.RDNSHandler != nil {
 			err = fh.RDNSHandler.Consume(e)
 			if err != nil {
